!============================================================================
! Name        : puregaugeupdate.f90
! Author      : Georg Bergner
! Version     :
! Copyright   :
! Description : Simple Fortran program to run pure gauge theory simulations.
!               Warning: several features (like random number generator)
!               not optimal for precise investigations.
!============================================================================

! matrix trace
double precision function realTraceM(mat)
 implicit none
 double complex mat(1:2,1:2)
 integer i
 realTraceM=0d0
 do i=1,2
  realTraceM=realTraceM+dble(mat(i,i))
 end do
end function realTraceM

! shifts of indices in up/down direction (periodic boundary conditions)
integer function shiftup(num,NS)
 implicit none
 integer num
 integer NS
 shiftup=num+1
 if(num==NS) then
  shiftup=1
 endif
end function shiftup

integer function shiftdown(num,NS)
 implicit none
 integer num
 integer NS
 shiftdown=num-1
 if(num==1) then
  shiftdown=NS
 endif
end function shiftdown

integer function toIndex(x,y,z,t,XS,YS,ZS,TS)
 implicit none
 integer x,y,z,t,XS,YS,ZS,TS
 toIndex=(x-1)+XS*((y-1)+YS*((z-1)+ZS*(t-1)))+1
end function toIndex

! random SU(2) matrix
SUBROUTINE SU2RANDOM(mat)
implicit none
double complex mat(1:2,1:2)
double precision rand(1:4)
double precision, parameter :: pi = 4 * atan(1.0_8)
double precision norm,phase,sinn,cosn
norm=2.0
do while(norm>1.0)
 call RANDOM_NUMBER(rand(1))
 call RANDOM_NUMBER(rand(2))
 call RANDOM_NUMBER(rand(3))
 norm=sqrt(rand(1)*rand(1)+rand(2)*rand(2)+rand(3)*rand(3))
end do
rand(1)=rand(1)/norm
rand(2)=rand(2)/norm
rand(3)=rand(3)/norm
call RANDOM_NUMBER(rand(4))
phase=2.0*Pi*rand(4)
cosn=cos(phase)
sinn=sin(phase)
mat(1,1)=cmplx(cosn,0.0)+cmplx(0.0,sinn*rand(3));
mat(1,2)=cmplx(0.0,sinn*rand(1))+cmplx(sinn*rand(2),0.0);
mat(2,2)=conjg(mat(1,1));
mat(2,1)=-conjg(mat(1,2));
end SUBROUTINE


program puregaugeupdate
    implicit none
    !   grid size and the parameter for the scan are fixed at compile time
    integer, parameter :: XS=6
    integer, parameter :: YS=6
    integer, parameter :: ZS=6
    integer, parameter :: TS=4
    integer, parameter :: VOL=XS*YS*ZS*TS
    double precision, parameter :: beta=2.0
    ! the thermalization has to be checked / adapted
    integer, parameter :: nthermal=200
    integer, parameter :: numconfigs=1000

    ! configuration
    double complex configuration(1:VOL,1:4,1:2,1:2)
    double complex tmpmat(1:2,1:2)
    double precision realTraceM

    ! lookup table for up and down neighbours
    integer uptable(1:VOL,1:4)
    integer downtable(1:VOL,1:4)
    integer toIndex,shiftup,shiftdown
    integer site,y,x,z,t,mu

    ! initialize up and down tables
    do x=1,XS
     do y=1,YS
      do z=1,ZS
       do t=1,TS
        site=toIndex(x,y,z,t,XS,YS,ZS,TS)
        uptable(site,1)=toIndex(shiftup(x,XS),y,z,t,XS,YS,ZS,TS)
        uptable(site,2)=toIndex(x,shiftup(y,YS),z,t,XS,YS,ZS,TS)
        uptable(site,3)=toIndex(x,y,shiftup(z,ZS),t,XS,YS,ZS,TS)
        uptable(site,4)=toIndex(x,y,z,shiftup(t,TS),XS,YS,ZS,TS)
        downtable(site,1)=toIndex(shiftdown(x,XS),y,z,t,XS,YS,ZS,TS)
        downtable(site,2)=toIndex(x,shiftdown(y,YS),z,t,XS,YS,ZS,TS)
        downtable(site,3)=toIndex(x,y,shiftdown(z,ZS),t,XS,YS,ZS,TS)
        downtable(site,4)=toIndex(x,y,z,shiftdown(t,TS),XS,YS,ZS,TS)
       end do
      end do
     end do
    end do

   ! set hot start
   do site=1,VOL
    do mu=1,4
    call SU2RANDOM(configuration(site,mu,:,:))
    end do
   end do

   ! TODO: add here the update algorithm and plaquette measurement

end program puregaugeupdate
