/*
 * su2metropolis.cpp
 *
 *  Created on: 15.01.2019
 *      Author: Georg Bergner
 *
 */

#include <stdio.h>
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <complex>
#include <boost/math/constants/constants.hpp>

#include "randomgenerator.h"
#include "lattice.h"
#include "configuration.h"
#include "Eigen/Eigen/Dense"

// definitions
typedef double Real;
typedef std::complex<Real> Complex;
static const int DIM = 4;
typedef Lattice<DIM> GLattice;
typedef Eigen::Matrix<Complex, 2,2> GaugeMatrix;
const Real pi=boost::math::constants::pi<Real >();
typedef Configuration<SiteLinks<GaugeMatrix,GLattice>,GLattice> GConfiguration;

#include "parameterparser.h"

// Random matrix SU(2)
inline void randomSu2(GaugeMatrix& randm, RandomNumberGenerator& rnd) {
	Real rand[3];
	Real norm(2.0);
	while(norm>1.0){
     rand[0]=rnd();
     rand[1]=rnd();
     rand[2]=rnd();
	 norm=sqrt(rand[0]*rand[0]+rand[1]*rand[1]+rand[2]*rand[2]);
	} // random direction on 3 sphere
	rand[0]/=norm;
	rand[1]/=norm;
	rand[2]/=norm;
	const Real phase(2.0*pi*rnd());
	const Real cosn(cos(phase));
	const Real sinn(sin(phase));
	randm(0,0)=Complex(cosn,0.0)+Complex(0.0,sinn*rand[2]);
	randm(0,1)=Complex(0.0,sinn*rand[0])+Complex(sinn*rand[1],0.0);
	randm(1,1)=conj(randm(0,0));
	randm(1,0)=-conj(randm(0,1));
}

template<class C>
void setHotStart(C& config, RandomNumberGenerator& rnd){
for(Index i(0);i<config.lattice().volume();i++){
	for(Index mu(0);mu<C::TSiteElement::numlinks;mu++){
		randomSu2(config[i][mu],rnd);
	}
}
}





int main(int argc, char** argv) {
	ModelParameter param;
	UpdateParameter updparam;
	GLattice::Coord sizes;
	CommandLineParser parser;

	// read parameter like param.beta from command line
	if (!parser.parseCommandLine(argc, argv, sizes, param, updparam)) {
		return (1);
	}

	// initialize random number generator, lattice, and configuration
	RandomNumberGenerator rand;
    GLattice lattice(sizes);
    GConfiguration configuration(lattice);
    setHotStart(configuration,rand);

    /* TODO: write updater at this point
     *Index site < lattice.volume()
     *Index mu < 4
     *Matrix u: configuration[site][mu]
     *trace: u.trace()
     *neighbour site: lattice.up(site,mu), lattice.down(site,mu)
     *adjoint: u.adjoint() (dagger)
     */
	return (0);
}



